/**
 * 
 */
package com.obdscope.servlet;

import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;
import java.util.StringTokenizer;

/**
 * A class that represents a OBDScope Remote logging client. Supports client 
 * initialization with an initialization string. Stores client parameter
 * identifiers, descriptions and the latest data. Presents client information
 * in HTML table form.
 * 
 * Copyright: Public domain.
 * 
 * @author www.obdscope.com
 *
 */
public class Client {

    private String id = "";
    private String nickName = "";
    private String logSeparator = ",";
    private char decimalSeparator;
    private List<String> parameterIds = null;
    private List<String> parameterDescriptions = null;
    private List<String> parameterValues = null;

    /**
     * Initialize client with the initialization string.
     * 
     * @param initString
     *            The initialization string containing client information, for
     *            example: OBDScope 1.12,.UxLbF6XTL,Racer 1
     * @throws Exception
     *             If the initString is malformed.
     */
    public Client(String initString) throws Exception {
        // System.out.println( "initString:" + initString);
        if (!initString.startsWith("OBDScope ")) {
            throw new Exception("Invalid initString");
        }

        // Parse client initialization parameters:
        this.logSeparator = this.logSeparator.replace(',', initString
                .charAt(13));
        this.decimalSeparator = initString.charAt(14);
        String idAndNick = initString.substring(15, initString.length());
        int splitPoint = idAndNick.indexOf(logSeparator);
        this.id = idAndNick.substring(0, splitPoint);
        this.nickName = idAndNick.substring(splitPoint + 1);
        parameterIds = new ArrayList();
        parameterDescriptions = new ArrayList();
        parameterValues = new ArrayList();
    }

    /**
     * 
     * @return Client ID
     */
    public String getId() {
        return id;
    }

    /**
     * Update the latest parameter identifiers, descriptions or values.
     * 
     * @param logRow
     *            Parameter data.
     */
    public void update(String logRow) {

        // Replace ;; strings with ;-; to match parameter values with
        // descriptions/identifiers:
        logRow = logRow.replaceAll(logSeparator + logSeparator, logSeparator
                + "-" + logSeparator);
        logRow = logRow.replaceAll(logSeparator + logSeparator, logSeparator
                + "-" + logSeparator);

        StringTokenizer tok = new StringTokenizer(logRow, logSeparator);
        if (tok.hasMoreElements()) {
            tok.nextElement(); // skip client id
        }
        if (parameterIds.size() == 0) {
            // The first time data is received from a client, it is the
            // parameter identifiers
            while (tok.hasMoreTokens()) {
                parameterIds.add(tok.nextToken());
            }
        } else if (parameterDescriptions.size() == 0) {
            // The second time data is received from a client, it is the
            // parameter descriptions
            while (tok.hasMoreTokens()) {
                String desc = tok.nextToken();
                parameterDescriptions.add(desc);
            }
        } else {
            // The third .. n time data is received from a client, it is the
            // parameter values
            parameterValues.clear(); // Store only the latest values
            while (tok.hasMoreTokens()) {
                String value = tok.nextToken();
                parameterValues.add(value);
            }
        }
    }

    /**
     * @return Client data in HTML table form.
     */
    public String toString() {
        String result = new String(
                "<p><table border=\"1\"><tr><td>ClientId</td><td>" + id
                        + "</td></tr><tr><td>Nickname</td><td>" + nickName
                        + "</td></tr>");
        Iterator descIterator = parameterDescriptions.iterator();
        Iterator valIterator = parameterValues.iterator();
        while (descIterator.hasNext() && valIterator.hasNext()) {
            result = result + "<tr><td>" + ((String) descIterator.next())
                    + "</td><td>"
                    + ((String) valIterator.next() + "</td></tr>");
        }
        return result + "</table></p>";
    }
}
